﻿/*
This source file is part of the RGBZ Video Camera project published in the paper

  Coherent Spatiotemporal Filtering, Upsampling and Rendering of RGBZ Videos
  Christian Richardt, Carsten Stoll, Neil A. Dodgson, Hans-Peter Seidel, Christian Theobalt
  Computer Graphics Forum (Proceedings of Eurographics), 31(2), May 2012
  <http://www.mpi-inf.mpg.de/resources/rgbz-camera/>

If you use our code, please cite our paper. Thanks.

Copyright (c) 2012 Christian Richardt <christian@richardt.name>


Code licence: GPL v3
----------------------------------------------------------------------------------------------------
This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation;
either version 3 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, see <http://www.gnu.org/licenses/>.

$Id: DistanceImageToPointCloud.frag 537 2012-04-04 15:35:42Z cr333 $
*/
#version 120
#extension GL_EXT_gpu_shader4 : enable

// input textures
uniform sampler2D input;

uniform vec2  fc; // focal lengths
uniform vec2  cc; // principal point

uniform float distanceScale; // depth rescaling
uniform int mapType; // depth or distance map?


void main(void)
{
	// look up texture size once and reuse it later
	vec2 size = vec2(textureSize2D(input, 0));
	
	float x = gl_FragCoord.x - 0.5;
	float y = gl_FragCoord.y - 0.5;
	
	// inv([Fx 0 Cx; 0 Fy Cy; 0 0 1]) = [iFx 0 iCx; 0 iFy iCy; 0 0 1]
	vec2 iF = 1.0 / fc;
	vec2 iC = -cc / fc;

	// TODO: maximum distance (here: 5.0f for SR4000) depends on modulation frequency
	float dist = texture2D(input, gl_FragCoord.xy / size).r * 5.0 * distanceScale;

	// inv(K) * [x y 1]' = [iFx 0 iCx; 0 iFy iCy; 0 0 1] * [x y 1]'
	vec3 ray = vec3(iF * (gl_FragCoord.xy - 0.5) + iC, 1.0);
	ray = ray * vec3(-1.0, -1.0, 1.0); // change of coordinate systems
	float x3 = iF.x * x + iC.x;
	float y3 = iF.y * y + iC.y;
	//float z3 = 1;

	if (mapType == 0) // distance map (e.g. SR4000)
	{
		// scale point to have desired distance along ray
		gl_FragColor = dist * vec4(normalize(ray), 1.0);
	}
	else if (mapType == 1) // depth map (e.g. Kinect)
	{
		// scale point to have desired scene depth
		gl_FragColor = dist * vec4(ray, length(ray));
	}

	// invalidate points further than magic distance set in PixelSearchReplace.frag
	if (dist > 1.49f)
		gl_FragColor.w = 0.0;
}
