﻿/*
This source file is part of the RGBZ Video Camera project published in the paper

  Coherent Spatiotemporal Filtering, Upsampling and Rendering of RGBZ Videos
  Christian Richardt, Carsten Stoll, Neil A. Dodgson, Hans-Peter Seidel, Christian Theobalt
  Computer Graphics Forum (Proceedings of Eurographics), 31(2), May 2012
  <http://www.mpi-inf.mpg.de/resources/rgbz-camera/>

If you use our code, please cite our paper. Thanks.

Copyright (c) 2012 Christian Richardt <christian@richardt.name>


Code licence: GPL v3
----------------------------------------------------------------------------------------------------
This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation;
either version 3 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, see <http://www.gnu.org/licenses/>.

$Id: StereoWarper.frag 537 2012-04-04 15:35:42Z cr333 $
*/
#extension GL_EXT_gpu_shader4 : enable

uniform sampler2D coordinates;
uniform sampler2D video;
uniform float shift;

// the vertex position, in viewing coordinates
varying vec3 v3;

varying vec2 coords;

void main()
{
	// transform from OpenGL coordinate system (x right, y down, z into the scene)
	// to SR4000 coordinate system (x left, y up, z into the scene), i.e. flip x & z
	gl_FragData[0] = vec4(-v3.x, v3.y, -v3.z, length(v3)); // new coordinate format: (x, y, z, distance)
	
	// textured mesh with colour video
	//gl_FragData[1] = texture2D(video, (coords + vec2(0.5, 0)) / vec2(textureSize2D(video, 0)));
	
	// pick furthest of two points
	vec2 size = vec2(textureSize2D(video, 0));
	vec2 lPos = vec2(floor(coords.x) - 1.0 + 0.5, coords.y + 0.5) / size;
	vec2 rPos = vec2(ceil(coords.x)  + 1.0 + 0.5, coords.y + 0.5) / size;
	vec4 lPoint = texture2D(coordinates, lPos);
	vec4 rPoint = texture2D(coordinates, rPos);
	
	// pick colour further away from the deeper side
	float dir = (lPoint.z > rPoint.z) ? -1.0 : 1.0;
	gl_FragData[1] = texture2D(video, (coords + vec2(dir * shift + 0.5, 0)) / size);
	
	// if there's not a big depth difference (<2cm), just use normal colour
	if(abs(lPoint.z - rPoint.z) < 0.02)
		gl_FragData[1] = texture2D(video, (coords + vec2(0.5, 0)) / vec2(textureSize2D(video, 0)));
	
	//if(abs(lPoint.z - rPoint.z) > 0.01) gl_FragData[1] = vec4(1,0,1,1);
}