﻿/*
This source file is part of the RGBZ Video Camera project published in the paper

  Coherent Spatiotemporal Filtering, Upsampling and Rendering of RGBZ Videos
  Christian Richardt, Carsten Stoll, Neil A. Dodgson, Hans-Peter Seidel, Christian Theobalt
  Computer Graphics Forum (Proceedings of Eurographics), 31(2), May 2012
  <http://www.mpi-inf.mpg.de/resources/rgbz-camera/>

If you use our code, please cite our paper. Thanks.

Copyright (c) 2012 Christian Richardt <christian@richardt.name>


Code licence: GPL v3
----------------------------------------------------------------------------------------------------
This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation;
either version 3 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, see <http://www.gnu.org/licenses/>.

$Id: TypeConverters.cs 537 2012-04-04 15:35:42Z cr333 $
*/
using System;
using System.ComponentModel;
using VideoLib.FlyCapture;

namespace RGBZcamera.Recorder
{
	public class FramerateConverter : EnumConverter
	{
		public FramerateConverter(Type type)
			: base(type)
		{
		}

		public override bool CanConvertTo(ITypeDescriptorContext context, Type destinationType)
		{
			if (destinationType == typeof(String))
				return true;

			return base.CanConvertTo(context, destinationType);
		}

		public override object ConvertTo(ITypeDescriptorContext context, System.Globalization.CultureInfo culture, object value, Type destinationType)
		{
			if (destinationType == typeof(String) && value is Framerates)
			{
				Framerates fps = (Framerates)value;
				return fps.ToString().Substring(1).Replace('_', '.') + " fps";
			}

			return base.ConvertTo(context, culture, value, destinationType);
		}

		public override bool CanConvertFrom(ITypeDescriptorContext context, Type sourceType)
		{
			if (sourceType == typeof(string))
				return true;

			return base.CanConvertFrom(context, sourceType);
		}

		public override object ConvertFrom(ITypeDescriptorContext context, System.Globalization.CultureInfo culture, object value)
		{
			string fps = value as string;
			string enumString = "_" + fps.Substring(0, fps.Length - 4).Replace('.', '_');
			return base.ConvertFrom(context, culture, enumString);
		}

		public override bool GetStandardValuesSupported(ITypeDescriptorContext context)
		{
			return true;
		}

		public override StandardValuesCollection GetStandardValues(ITypeDescriptorContext context)
		{
			return new StandardValuesCollection(new Framerates[] {
				Framerates._1_875,
				Framerates._3_75,
				Framerates._7_5,
				Framerates._15,
				Framerates._30
			});
		}
	}

	public class ResolutionConverter : Int32Converter
	{
		public override bool GetStandardValuesSupported(ITypeDescriptorContext context)
		{
			return true;
		}

		public override StandardValuesCollection GetStandardValues(ITypeDescriptorContext context)
		{
			return new StandardValuesCollection(new int[] { 160, 320, 640, 800, 1024 });
		}
	}
}