/*
This source file is part of the RGBZ Video Camera project published in the paper

  Coherent Spatiotemporal Filtering, Upsampling and Rendering of RGBZ Videos
  Christian Richardt, Carsten Stoll, Neil A. Dodgson, Hans-Peter Seidel, Christian Theobalt
  Computer Graphics Forum (Proceedings of Eurographics), 31(2), May 2012
  <http://www.mpi-inf.mpg.de/resources/rgbz-camera/>

If you use our code, please cite our paper. Thanks.

Copyright (c) 2012 Christian Richardt <christian@richardt.name>


Code licence: GPL v3
----------------------------------------------------------------------------------------------------
This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation;
either version 3 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, see <http://www.gnu.org/licenses/>.

$Id: BitmapFrames.cpp 559 2012-09-12 11:41:46Z cr333 $
*/
#include "BitmapFrames.h"
#include <stdio.h>
#include <string.h>

// fopen_s is Microsoft-specific, so hack a version for other compilers.
#ifndef _MSC_VER
bool fopen_s(FILE **f, const char *name, const char *mode) {
	*f = fopen(name, mode);
	/* Can't be sure about 1-to-1 mapping of errno and MS' errno_t */
	return !*f;
}
#endif

namespace BitmapFrames
{
	// Saves an image to a raw BitmapFrame on disk.
	bool saveRawBitmapFrame(const void* data, const RawHeader& header, const char* filename)
	{
		// open file
		FILE* fh = NULL;
		if(fopen_s(&fh, filename, "wb") != 0)
			return false;

		// write magic bytes
		if(fwrite("\x0B" "BitmapFrame", 1, 12, fh) != 12)
			return false;

		// write header
		if(fwrite(&header, sizeof(RawHeader), 1, fh) != 1)
			return false;

		// write pixels
		if(fwrite(data, 1, header.stride * header.height, fh) != header.stride * header.height)
			return false;

		// close file
		if(fclose(fh) != 0)
			return false;

		// all done
		return true;
	}


	// Reads a raw BitmapFrame from disk into memory.
	//
	// This function allocates memory for the data to be read using "new",
	// so to free the memory, call "delete" on the pointer.
	bool readRawBitmapFrame(const char* filename, RawHeader& header, void** data)
	{
		// open file
		FILE* fh = NULL;
		if(fopen_s(&fh, filename, "rb") != 0)
			return false;

		// read magic bytes
		char buf[13]; memset(buf, 0, 13);
		if(fread(&buf, 1, 12, fh) != 12)
			return false;

		// check magic bytes
		if(strcmp(buf, "\x0B" "BitmapFrame") != 0)
			return false;

		// read header
		if(fread(&header, sizeof(RawHeader), 1, fh) != 1)
			return false;

		// read pixels
		*data = new unsigned char[header.stride * header.height];
		if(fread(*data, 1, header.stride * header.height, fh) != header.stride * header.height)
			return false;

		// close file
		if(fclose(fh) != 0)
			return false;

		// all done
		return true;
	}
}