// LibAvWrap -- .NET wrapper for the libav* libraries
// --------------------------------------------------
// By Christian Richardt <christian@richardt.name>
// (based on AvDn 0.7, www.informikon.com/tutorials/)
//
// $Id: Enums.h 262 2010-07-28 13:25:26Z cr333 $

#pragma once

using namespace System;
using namespace System::Text;
using namespace System::Runtime::InteropServices;

extern "C" {
#include "libavcodec/avcodec.h"
#include "libavformat/avformat.h"
#include "libavutil/avutil.h"
#include "libswscale/swscale.h"
}

///<summary>Wrapper for some lavc and lavf structures.</summary>
namespace LibAvWrap
{
	///<summary>Contains some constants from the libav* libraries.</summary>
	public ref class Av
	{
	public:
		static const int AvFormatNoFile = AVFMT_NOFILE;
		static const int AvRawPicture   = AVFMT_RAWPICTURE;

		static const int UrlReadOnly  = URL_RDONLY;
		static const int UrlWriteOnly = URL_WRONLY;
		static const int UrlReadWrite = URL_RDWR;
	};

	///<summary>
	///These flags can be passed in CodecContext.Flags before initialisation.
	///<remark>Copied verbatim from avcodec.h v52.79.1</remark>
	///</summary>
	[Flags] public enum class CodecFlags {
		QScale              = CODEC_FLAG_QSCALE,  ///< Use fixed qscale.
		_4MV                = CODEC_FLAG_4MV,  ///< 4 MV per MB allowed / advanced prediction for H.263.
		QPel                = CODEC_FLAG_QPEL,  ///< Use qpel MC.
		GMC                 = CODEC_FLAG_GMC,  ///< Use GMC.
		MV0                 = CODEC_FLAG_MV0,  ///< Always try a MB with MV=<0,0>.
		Part                = CODEC_FLAG_PART,  ///< Use data partitioning.
		/**
		 * The parent program guarantees that the input for B-frames containing
		 * streams is not written to for at least s->max_b_frames+1 frames, if
		 * this is not set the input will be copied.
		 */
		InputPreserved      = CODEC_FLAG_INPUT_PRESERVED,
		Pass1               = CODEC_FLAG_PASS1,   ///< Use internal 2pass ratecontrol in first pass mode.
		Pass2               = CODEC_FLAG_PASS2,   ///< Use internal 2pass ratecontrol in second pass mode.
		ExternHuff          = CODEC_FLAG_EXTERN_HUFF,   ///< Use external Huffman table (for MJPEG).
		Gray                = CODEC_FLAG_GRAY,   ///< Only decode/encode grayscale.
		EmuEdge             = CODEC_FLAG_EMU_EDGE,   ///< Don't draw edges.
		PSNR                = CODEC_FLAG_PSNR,   ///< error[?] variables will be set during encoding.
		Truncated           = CODEC_FLAG_TRUNCATED, /** Input bitstream might be truncated at a random
		                                                  location instead of only at frame boundaries. */
		NormalizeAQP        = CODEC_FLAG_NORMALIZE_AQP, ///< Normalize adaptive quantization.
		InterlacedDCT       = CODEC_FLAG_INTERLACED_DCT, ///< Use interlaced DCT.
		LowDelay            = CODEC_FLAG_LOW_DELAY, ///< Force low delay.
		AltScan             = CODEC_FLAG_ALT_SCAN, ///< Use alternate scan.
		GlobalHeader        = CODEC_FLAG_GLOBAL_HEADER, ///< Place global headers in extradata instead of every keyframe.
		Bitexact            = CODEC_FLAG_BITEXACT, ///< Use only bitexact stuff (except (I)DCT).
		/* Fx : Flag for h263+ extra options */
		ACPred              = CODEC_FLAG_AC_PRED, ///< H.263 advanced intra coding / MPEG-4 AC prediction
		H263P_UMV           = CODEC_FLAG_H263P_UMV, ///< unlimited motion vector
		CBP_RD              = CODEC_FLAG_CBP_RD, ///< Use rate distortion optimization for cbp.
		QP_RD               = CODEC_FLAG_QP_RD, ///< Use rate distortion optimization for qp selectioon.
		H263P_AIV           = CODEC_FLAG_H263P_AIV, ///< H.263 alternative inter VLC
		OBMC                = CODEC_FLAG_OBMC, ///< OBMC
		LoopFilter          = CODEC_FLAG_LOOP_FILTER, ///< loop filter
		H263P_SliceStruct   = CODEC_FLAG_H263P_SLICE_STRUCT,
		InterlacedME        = CODEC_FLAG_INTERLACED_ME, ///< interlaced motion estimation
		SVCDScanOffset      = CODEC_FLAG_SVCD_SCAN_OFFSET, ///< Will reserve space for SVCD scan offset user data.
		ClosedGop           = CODEC_FLAG_CLOSED_GOP,
	};

	///<summary>
	///These additional flags can be passed in CodecContext.Flags before initialisation.
	///<remark>Copied verbatim from avcodec.h v52.79.1</remark>
	///</summary>
	[Flags] public enum class CodecFlags2 {
		Fast              = CODEC_FLAG2_FAST,                ///< Allow non spec compliant speedup tricks.
		StrictGop         = CODEC_FLAG2_STRICT_GOP,          ///< Strictly enforce GOP size.
		NoOutput          = CODEC_FLAG2_NO_OUTPUT,           ///< Skip bitstream encoding.
		LocalHeader       = CODEC_FLAG2_LOCAL_HEADER,        ///< Place global headers at every keyframe instead of in extradata.
		BPyramid          = CODEC_FLAG2_BPYRAMID,            ///< H.264 allow B-frames to be used as references.
		WPred             = CODEC_FLAG2_WPRED,               ///< H.264 weighted biprediction for B-frames
		MixedRefs         = CODEC_FLAG2_MIXED_REFS,          ///< H.264 one reference per partition, as opposed to one reference per macroblock
		_8X8DCT           = CODEC_FLAG2_8X8DCT,              ///< H.264 high profile 8x8 transform
		FastSkip          = CODEC_FLAG2_FASTPSKIP,           ///< H.264 fast pskip
		AUD               = CODEC_FLAG2_AUD,                 ///< H.264 access unit delimiters
		BRDO              = CODEC_FLAG2_BRDO,                ///< B-frame rate-distortion optimization
		IntraVLC          = CODEC_FLAG2_INTRA_VLC,           ///< Use MPEG-2 intra VLC table.
		MEMCOnly          = CODEC_FLAG2_MEMC_ONLY,           ///< Only do ME/MC (I frames -> ref, P frame -> ME+MC).
		DropFrameTimecode = CODEC_FLAG2_DROP_FRAME_TIMECODE, ///< timecode is in drop frame format.
		SkipRD            = CODEC_FLAG2_SKIP_RD,             ///< RD optimal MB level residual skipping
		Chunks            = CODEC_FLAG2_CHUNKS,              ///< Input bitstream might be truncated at a packet boundaries instead of only at frame boundaries.
		NonLinearQuant    = CODEC_FLAG2_NON_LINEAR_QUANT,    ///< Use MPEG-2 nonlinear quantizer.
		BitReservoir      = CODEC_FLAG2_BIT_RESERVOIR,       ///< Use a bit reservoir when encoding if possible
		MBTree            = CODEC_FLAG2_MBTREE,              ///< Use macroblock tree ratecontrol (x264 only)
		Psy               = CODEC_FLAG2_PSY,                 ///< Use psycho visual optimizations.
		SSIM              = CODEC_FLAG2_SSIM,                ///< Compute SSIM during encoding, error[] values are undefined.
		IntraRefesh       = CODEC_FLAG2_INTRA_REFRESH,       ///< Use periodic insertion of intra blocks instead of keyframes.
	};

	///<summary>
	///Identifies the syntax and semantics of the bitstream.
	///<remark>Copied verbatim from avcodec.h v52.79.1</remark>
	///</summary>
	public enum class CodecID {
		NONE = CODEC_ID_NONE,

		/* video codecs */
		MPEG1VIDEO = CODEC_ID_MPEG1VIDEO,
		MPEG2VIDEO = CODEC_ID_MPEG2VIDEO, ///< preferred ID for MPEG-1/2 video decoding
		MPEG2VIDEO_XVMC = CODEC_ID_MPEG2VIDEO_XVMC,
		H261 = CODEC_ID_H261,
		H263 = CODEC_ID_H263,
		RV10 = CODEC_ID_RV10,
		RV20 = CODEC_ID_RV20,
		MJPEG = CODEC_ID_MJPEG,
		MJPEGB = CODEC_ID_MJPEGB,
		LJPEG = CODEC_ID_LJPEG,
		SP5X = CODEC_ID_SP5X,
		JPEGLS = CODEC_ID_JPEGLS,
		MPEG4 = CODEC_ID_MPEG4,
		RAWVIDEO = CODEC_ID_RAWVIDEO,
		MSMPEG4V1 = CODEC_ID_MSMPEG4V1,
		MSMPEG4V2 = CODEC_ID_MSMPEG4V2,
		MSMPEG4V3 = CODEC_ID_MSMPEG4V3,
		WMV1 = CODEC_ID_WMV1,
		WMV2 = CODEC_ID_WMV2,
		H263P = CODEC_ID_H263P,
		H263I = CODEC_ID_H263I,
		FLV1 = CODEC_ID_FLV1,
		SVQ1 = CODEC_ID_SVQ1,
		SVQ3 = CODEC_ID_SVQ3,
		DVVIDEO = CODEC_ID_DVVIDEO,
		HUFFYUV = CODEC_ID_HUFFYUV,
		CYUV = CODEC_ID_CYUV,
		H264 = CODEC_ID_H264,
		INDEO3 = CODEC_ID_INDEO3,
		VP3 = CODEC_ID_VP3,
		THEORA = CODEC_ID_THEORA,
		ASV1 = CODEC_ID_ASV1,
		ASV2 = CODEC_ID_ASV2,
		FFV1 = CODEC_ID_FFV1,
		_4XM = CODEC_ID_4XM,
		VCR1 = CODEC_ID_VCR1,
		CLJR = CODEC_ID_CLJR,
		MDEC = CODEC_ID_MDEC,
		ROQ = CODEC_ID_ROQ,
		INTERPLAY_VIDEO = CODEC_ID_INTERPLAY_VIDEO,
		XAN_WC3 = CODEC_ID_XAN_WC3,
		XAN_WC4 = CODEC_ID_XAN_WC4,
		RPZA = CODEC_ID_RPZA,
		CINEPAK = CODEC_ID_CINEPAK,
		WS_VQA = CODEC_ID_WS_VQA,
		MSRLE = CODEC_ID_MSRLE,
		MSVIDEO1 = CODEC_ID_MSVIDEO1,
		IDCIN = CODEC_ID_IDCIN,
		_8BPS = CODEC_ID_8BPS,
		SMC = CODEC_ID_SMC,
		FLIC = CODEC_ID_FLIC,
		TRUEMOTION1 = CODEC_ID_TRUEMOTION1,
		VMDVIDEO = CODEC_ID_VMDVIDEO,
		MSZH = CODEC_ID_MSZH,
		ZLIB = CODEC_ID_ZLIB,
		QTRLE = CODEC_ID_QTRLE,
		SNOW = CODEC_ID_SNOW,
		TSCC = CODEC_ID_TSCC,
		ULTI = CODEC_ID_ULTI,
		QDRAW = CODEC_ID_QDRAW,
		VIXL = CODEC_ID_VIXL,
		QPEG = CODEC_ID_QPEG,
// #if LIBAVCODEC_VERSION_MAJOR < 53
		XVID = CODEC_ID_XVID,
// #endif
		PNG = CODEC_ID_PNG,
		PPM = CODEC_ID_PPM,
		PBM = CODEC_ID_PBM,
		PGM = CODEC_ID_PGM,
		PGMYUV = CODEC_ID_PGMYUV,
		PAM = CODEC_ID_PAM,
		FFVHUFF = CODEC_ID_FFVHUFF,
		RV30 = CODEC_ID_RV30,
		RV40 = CODEC_ID_RV40,
		VC1 = CODEC_ID_VC1,
		WMV3 = CODEC_ID_WMV3,
		LOCO = CODEC_ID_LOCO,
		WNV1 = CODEC_ID_WNV1,
		AASC = CODEC_ID_AASC,
		INDEO2 = CODEC_ID_INDEO2,
		FRAPS = CODEC_ID_FRAPS,
		TRUEMOTION2 = CODEC_ID_TRUEMOTION2,
		BMP = CODEC_ID_BMP,
		CSCD = CODEC_ID_CSCD,
		MMVIDEO = CODEC_ID_MMVIDEO,
		ZMBV = CODEC_ID_ZMBV,
		AVS = CODEC_ID_AVS,
		SMACKVIDEO = CODEC_ID_SMACKVIDEO,
		NUV = CODEC_ID_NUV,
		KMVC = CODEC_ID_KMVC,
		FLASHSV = CODEC_ID_FLASHSV,
		CAVS = CODEC_ID_CAVS,
		JPEG2000 = CODEC_ID_JPEG2000,
		VMNC = CODEC_ID_VMNC,
		VP5 = CODEC_ID_VP5,
		VP6 = CODEC_ID_VP6,
		VP6F = CODEC_ID_VP6F,
		TARGA = CODEC_ID_TARGA,
		DSICINVIDEO = CODEC_ID_DSICINVIDEO,
		TIERTEXSEQVIDEO = CODEC_ID_TIERTEXSEQVIDEO,
		TIFF = CODEC_ID_TIFF,
		GIF = CODEC_ID_GIF,
		FFH264 = CODEC_ID_FFH264,
		DXA = CODEC_ID_DXA,
		DNXHD = CODEC_ID_DNXHD,
		THP = CODEC_ID_THP,
		SGI = CODEC_ID_SGI,
		C93 = CODEC_ID_C93,
		BETHSOFTVID = CODEC_ID_BETHSOFTVID,
		PTX = CODEC_ID_PTX,
		TXD = CODEC_ID_TXD,
		VP6A = CODEC_ID_VP6A,
		AMV = CODEC_ID_AMV,
		VB = CODEC_ID_VB,
		PCX = CODEC_ID_PCX,
		SUNRAST = CODEC_ID_SUNRAST,
		INDEO4 = CODEC_ID_INDEO4,
		INDEO5 = CODEC_ID_INDEO5,
		MIMIC = CODEC_ID_MIMIC,
		RL2 = CODEC_ID_RL2,
		_8SVX_EXP = CODEC_ID_8SVX_EXP,
		_8SVX_FIB = CODEC_ID_8SVX_FIB,
		ESCAPE124 = CODEC_ID_ESCAPE124,
		DIRAC = CODEC_ID_DIRAC,
		BFI = CODEC_ID_BFI,
		CMV = CODEC_ID_CMV,
		MOTIONPIXELS = CODEC_ID_MOTIONPIXELS,
		TGV = CODEC_ID_TGV,
		TGQ = CODEC_ID_TGQ,
		TQI = CODEC_ID_TQI,
		AURA = CODEC_ID_AURA,
		AURA2 = CODEC_ID_AURA2,
		V210X = CODEC_ID_V210X,
		TMV = CODEC_ID_TMV,
		V210 = CODEC_ID_V210,
		DPX = CODEC_ID_DPX,
		MAD = CODEC_ID_MAD,
		FRWU = CODEC_ID_FRWU,
		FLASHSV2 = CODEC_ID_FLASHSV2,
		CDGRAPHICS = CODEC_ID_CDGRAPHICS,
		R210 = CODEC_ID_R210,
		ANM = CODEC_ID_ANM,
		BINKVIDEO = CODEC_ID_BINKVIDEO,
		IFF_ILBM = CODEC_ID_IFF_ILBM,
		IFF_BYTERUN1 = CODEC_ID_IFF_BYTERUN1,
		KGV1 = CODEC_ID_KGV1,
		YOP = CODEC_ID_YOP,
		VP8 = CODEC_ID_VP8,
		PICTOR = CODEC_ID_PICTOR,

		/* various PCM "codecs" */
		PCM_S16LE = CODEC_ID_PCM_S16LE,
		PCM_S16BE = CODEC_ID_PCM_S16BE,
		PCM_U16LE = CODEC_ID_PCM_U16LE,
		PCM_U16BE = CODEC_ID_PCM_U16BE,
		PCM_S8 = CODEC_ID_PCM_S8,
		PCM_U8 = CODEC_ID_PCM_U8,
		PCM_MULAW = CODEC_ID_PCM_MULAW,
		PCM_ALAW = CODEC_ID_PCM_ALAW,
		PCM_S32LE = CODEC_ID_PCM_S32LE,
		PCM_S32BE = CODEC_ID_PCM_S32BE,
		PCM_U32LE = CODEC_ID_PCM_U32LE,
		PCM_U32BE = CODEC_ID_PCM_U32BE,
		PCM_S24LE = CODEC_ID_PCM_S24LE,
		PCM_S24BE = CODEC_ID_PCM_S24BE,
		PCM_U24LE = CODEC_ID_PCM_U24LE,
		PCM_U24BE = CODEC_ID_PCM_U24BE,
		PCM_S24DAUD = CODEC_ID_PCM_S24DAUD,
		PCM_ZORK = CODEC_ID_PCM_ZORK,
		PCM_S16LE_PLANAR = CODEC_ID_PCM_S16LE_PLANAR,
		PCM_DVD = CODEC_ID_PCM_DVD,
		PCM_F32BE = CODEC_ID_PCM_F32BE,
		PCM_F32LE = CODEC_ID_PCM_F32LE,
		PCM_F64BE = CODEC_ID_PCM_F64BE,
		PCM_F64LE = CODEC_ID_PCM_F64LE,
		PCM_BLURAY = CODEC_ID_PCM_BLURAY,

		/* various ADPCM codecs */
		ADPCM_IMA_QT = CODEC_ID_ADPCM_IMA_QT,
		ADPCM_IMA_WAV = CODEC_ID_ADPCM_IMA_WAV,
		ADPCM_IMA_DK3 = CODEC_ID_ADPCM_IMA_DK3,
		ADPCM_IMA_DK4 = CODEC_ID_ADPCM_IMA_DK4,
		ADPCM_IMA_WS = CODEC_ID_ADPCM_IMA_WS,
		ADPCM_IMA_SMJPEG = CODEC_ID_ADPCM_IMA_SMJPEG,
		ADPCM_MS = CODEC_ID_ADPCM_MS,
		ADPCM_4XM = CODEC_ID_ADPCM_4XM,
		ADPCM_XA = CODEC_ID_ADPCM_XA,
		ADPCM_ADX = CODEC_ID_ADPCM_ADX,
		ADPCM_EA = CODEC_ID_ADPCM_EA,
		ADPCM_G726 = CODEC_ID_ADPCM_G726,
		ADPCM_CT = CODEC_ID_ADPCM_CT,
		ADPCM_SWF = CODEC_ID_ADPCM_SWF,
		ADPCM_YAMAHA = CODEC_ID_ADPCM_YAMAHA,
		ADPCM_SBPRO_4 = CODEC_ID_ADPCM_SBPRO_4,
		ADPCM_SBPRO_3 = CODEC_ID_ADPCM_SBPRO_3,
		ADPCM_SBPRO_2 = CODEC_ID_ADPCM_SBPRO_2,
		ADPCM_THP = CODEC_ID_ADPCM_THP,
		ADPCM_IMA_AMV = CODEC_ID_ADPCM_IMA_AMV,
		ADPCM_EA_R1 = CODEC_ID_ADPCM_EA_R1,
		ADPCM_EA_R3 = CODEC_ID_ADPCM_EA_R3,
		ADPCM_EA_R2 = CODEC_ID_ADPCM_EA_R2,
		ADPCM_IMA_EA_SEAD = CODEC_ID_ADPCM_IMA_EA_SEAD,
		ADPCM_IMA_EA_EACS = CODEC_ID_ADPCM_IMA_EA_EACS,
		ADPCM_EA_XAS = CODEC_ID_ADPCM_EA_XAS,
		ADPCM_EA_MAXIS_XA = CODEC_ID_ADPCM_EA_MAXIS_XA,
		ADPCM_IMA_ISS = CODEC_ID_ADPCM_IMA_ISS,

		/* AMR */
		AMR_NB = CODEC_ID_AMR_NB,
		AMR_WB = CODEC_ID_AMR_WB,

		/* RealAudio codecs*/
		RA_144 = CODEC_ID_RA_144,
		RA_288 = CODEC_ID_RA_288,

		/* various DPCM codecs */
		ROQ_DPCM = CODEC_ID_ROQ_DPCM,
		INTERPLAY_DPCM = CODEC_ID_INTERPLAY_DPCM,
		XAN_DPCM = CODEC_ID_XAN_DPCM,
		SOL_DPCM = CODEC_ID_SOL_DPCM,

		/* audio codecs */
		MP2 = CODEC_ID_MP2,
		MP3 = CODEC_ID_MP3, ///< preferred ID for decoding MPEG audio layer 1, 2 or 3
		AAC = CODEC_ID_AAC,
		AC3 = CODEC_ID_AC3,
		DTS = CODEC_ID_DTS,
		VORBIS = CODEC_ID_VORBIS,
		DVAUDIO = CODEC_ID_DVAUDIO,
		WMAV1 = CODEC_ID_WMAV1,
		WMAV2 = CODEC_ID_WMAV2,
		MACE3 = CODEC_ID_MACE3,
		MACE6 = CODEC_ID_MACE6,
		VMDAUDIO = CODEC_ID_VMDAUDIO,
		SONIC = CODEC_ID_SONIC,
		SONIC_LS = CODEC_ID_SONIC_LS,
		FLAC = CODEC_ID_FLAC,
		MP3ADU = CODEC_ID_MP3ADU,
		MP3ON4 = CODEC_ID_MP3ON4,
		SHORTEN = CODEC_ID_SHORTEN,
		ALAC = CODEC_ID_ALAC,
		WESTWOOD_SND1 = CODEC_ID_WESTWOOD_SND1,
		GSM = CODEC_ID_GSM, ///< as in Berlin toast format
		QDM2 = CODEC_ID_QDM2,
		COOK = CODEC_ID_COOK,
		TRUESPEECH = CODEC_ID_TRUESPEECH,
		TTA = CODEC_ID_TTA,
		SMACKAUDIO = CODEC_ID_SMACKAUDIO,
		QCELP = CODEC_ID_QCELP,
		WAVPACK = CODEC_ID_WAVPACK,
		DSICINAUDIO = CODEC_ID_DSICINAUDIO,
		IMC = CODEC_ID_IMC,
		MUSEPACK7 = CODEC_ID_MUSEPACK7,
		MLP = CODEC_ID_MLP,
		GSM_MS = CODEC_ID_GSM_MS, /* as found in WAV */
		ATRAC3 = CODEC_ID_ATRAC3,
		VOXWARE = CODEC_ID_VOXWARE,
		APE = CODEC_ID_APE,
		NELLYMOSER = CODEC_ID_NELLYMOSER,
		MUSEPACK8 = CODEC_ID_MUSEPACK8,
		SPEEX = CODEC_ID_SPEEX,
		WMAVOICE = CODEC_ID_WMAVOICE,
		WMAPRO = CODEC_ID_WMAPRO,
		WMALOSSLESS = CODEC_ID_WMALOSSLESS,
		ATRAC3P = CODEC_ID_ATRAC3P,
		EAC3 = CODEC_ID_EAC3,
		SIPR = CODEC_ID_SIPR,
		MP1 = CODEC_ID_MP1,
		TWINVQ = CODEC_ID_TWINVQ,
		TRUEHD = CODEC_ID_TRUEHD,
		MP4ALS = CODEC_ID_MP4ALS,
		ATRAC1 = CODEC_ID_ATRAC1,
		BINKAUDIO_RDFT = CODEC_ID_BINKAUDIO_RDFT,
		BINKAUDIO_DCT = CODEC_ID_BINKAUDIO_DCT,

		/* subtitle codecs */
		DVD_SUBTITLE = CODEC_ID_DVD_SUBTITLE,
		DVB_SUBTITLE = CODEC_ID_DVB_SUBTITLE,
		TEXT = CODEC_ID_TEXT,  ///< raw UTF-8 text
		XSUB = CODEC_ID_XSUB,
		SSA = CODEC_ID_SSA,
		MOV_TEXT = CODEC_ID_MOV_TEXT,
		HDMV_PGS_SUBTITLE = CODEC_ID_HDMV_PGS_SUBTITLE,
		DVB_TELETEXT = CODEC_ID_DVB_TELETEXT,

		/* other specific kind of codecs (generally used for attachments) */
		TTF = CODEC_ID_TTF,

		PROBE = CODEC_ID_PROBE, ///< codec_id is not known (like NONE = CODEC_ID_NONE) but lavf should attempt to identify it

		MPEG2TS = CODEC_ID_MPEG2TS, /**< _FAKE_ codec to indicate a raw MPEG-2 TS
		                              * stream (only used by libavformat) */
	};

	///<summary>Codec Type.</summary>
	///<remark>From avutil.h v50.20</remark>
	public enum class MediaType {
		Unknown    = AVMEDIA_TYPE_UNKNOWN,
		Video      = AVMEDIA_TYPE_VIDEO,
		Audio      = AVMEDIA_TYPE_AUDIO,
		Data       = AVMEDIA_TYPE_DATA,
		Subtitle   = AVMEDIA_TYPE_SUBTITLE,
		Attachment = AVMEDIA_TYPE_ATTACHMENT,
		NB         = AVMEDIA_TYPE_NB,
	};

	// NB: items are reordered by similarity.
	///<summary>
	///Pixel Format.
	///</summary>
	///<remark>
	/// PIX_FMT_RGB32 is handled in an endian-specific manner. A RGBA
	/// color is put together as:
	///  (A << 24) | (R << 16) | (G << 8) | B
	/// This is stored as BGRA on little endian CPU architectures and ARGB on
 	/// big endian CPUs.
 	///
 	/// When the pixel format is palettized RGB (PIX_FMT_PAL8), the palettized
 	/// image data is stored in AVFrame.data[0]. The palette is transported in
 	/// AVFrame.data[1] and, is 1024 bytes long (256 4-byte entries) and is
 	/// formatted the same as in PIX_FMT_RGB32 described above (i.e., it is
 	/// also endian-specific). Note also that the individual RGB palette
 	/// components stored in AVFrame.data[1] should be in the range 0..255.
 	/// This is important as many custom PAL8 video codecs that were designed
 	/// to run on the IBM VGA graphics adapter use 6-bit palette components.
	///
	/// For all the 8bit per pixel formats, an RGB32 palette is in data[1] like
	/// for pal8. This palette is filled in automatically by the function
	/// allocating the picture.
	///
	/// Based on libavutil/pixfmt.h.
	///</remark>
	public enum class PixelFormat {
		None            = PIX_FMT_NONE,

		Gray8           = PIX_FMT_GRAY8,       ///<        Y        ,  8bpp
		Gray16          = PIX_FMT_GRAY16,      ///<        Y        , 16bpp, in cpu endianness
		Gray16B         = PIX_FMT_GRAY16BE,    ///<        Y        , 16bpp, big-endian
		Gray16L         = PIX_FMT_GRAY16LE,    ///<        Y        , 16bpp, little-endian
		MonoWhite       = PIX_FMT_MONOWHITE,   ///<        Y        ,  1bpp, 0 is white, 1 is black
		MonoBlack       = PIX_FMT_MONOBLACK,   ///<        Y        ,  1bpp, 0 is black, 1 is white

		RGB4            = PIX_FMT_RGB4,        ///< Packed RGB 1:2:1,  4bpp, (msb)1R 2G 1B(lsb)
		RGB4Byte        = PIX_FMT_RGB4_BYTE,   ///< Packed RGB 1:2:1,  8bpp, (msb)1R 2G 1B(lsb)
		RGB8            = PIX_FMT_RGB8,        ///< Packed RGB 3:3:2,  8bpp, (msb)2R 3G 3B(lsb)
		RGB444          = PIX_FMT_RGB444,      ///< Packed RGB 4:4:4, 16bpp, (msb)4A 4R 4G 4B(lsb), in cpu endianness, most significant bits to 0
		RGB555          = PIX_FMT_RGB555,      ///< Packed RGB 5:5:5, 16bpp, (msb)1A 5R 5G 5B(lsb), in cpu endianness, most significant bit to 0
		RGB565          = PIX_FMT_RGB565,      ///< Packed RGB 5:6:5, 16bpp, (msb)   5R 6G 5B(lsb), in cpu endianness
		RGB24           = PIX_FMT_RGB24,       ///< Packed RGB 8:8:8, 24bpp, RGBRGB...
		RGB32           = PIX_FMT_RGB32,       ///< Packed RGB 8:8:8, 32bpp, (msb)8A 8R 8G 8B(lsb), in cpu endianness
		RGB32_1         = PIX_FMT_RGB32_1,     ///< Packed RGB 8:8:8, 32bpp, (msb)8R 8G 8B 8A(lsb), in cpu endianness
		RGB48           = PIX_FMT_RGB48,       ///< Packed RGB 16:16:16, 48bpp, 16R, 16G, 16B, the 2-byte value for each R/G/B component is stored in cpu endianness

		BGR4            = PIX_FMT_BGR4,        ///< Packed RGB 1:2:1,  4bpp, (msb)1B 2G 1R(lsb)
		BGR4Byte        = PIX_FMT_BGR4_BYTE,   ///< Packed RGB 1:2:1,  8bpp, (msb)1B 2G 1R(lsb)
		BGR8            = PIX_FMT_BGR8,        ///< Packed RGB 3:3:2,  8bpp, (msb)2B 3G 3R(lsb)
		BGR444          = PIX_FMT_BGR444,      ///< Packed BGR 4:4:4, 16bpp, (msb)4A 4B 4G 4R(lsb), in cpu endianness, most significant bits to 1
		BGR555          = PIX_FMT_BGR555,      ///< Packed RGB 5:5:5, 16bpp, (msb)1A 5B 5G 5R(lsb), in cpu endianness, most significant bit to 1
		BGR565          = PIX_FMT_BGR565,      ///< Packed RGB 5:6:5, 16bpp, (msb)   5B 6G 5R(lsb), in cpu endianness
		BGR24           = PIX_FMT_BGR24,       ///< Packed RGB 8:8:8, 24bpp, BGRBGR...
		BGR32           = PIX_FMT_BGR32,       ///< Packed RGB 8:8:8, 32bpp, (msb)8A 8B 8G 8R(lsb), in cpu endianness
		BGR32_1         = PIX_FMT_BGR32_1,     ///< Packed RGB 8:8:8, 32bpp, (msb)8B 8G 8R 8A(lsb), in cpu endianness

		Pal8            = PIX_FMT_PAL8,        ///< 8 bit with PIX_FMT_RGB32 palette

		YUV410P         = PIX_FMT_YUV410P,     ///< Planar YUV 4:1:0,  9bpp, (1 Cr & Cb sample per 4x4 Y samples)

		YUV411P         = PIX_FMT_YUV411P,     ///< Planar YUV 4:1:1, 12bpp, (1 Cr & Cb sample per 4x1 Y samples)
		UYYVYY411       = PIX_FMT_UYYVYY411,   ///< Packed YUV 4:1:1, 12bpp, Cb Y0 Y1 Cr Y2 Y3

		YUV420P         = PIX_FMT_YUV420P,     ///< Planar YUV 4:2:0, 12bpp, (1 Cr & Cb sample per 2x2 Y samples)
		YUVJ420P        = PIX_FMT_YUVJ420P,    ///< Planar YUV 4:2:0, 12bpp, full scale (jpeg)
		YUVA420P        = PIX_FMT_YUVA420P,    ///< Planar YUV 4:2:0, 20bpp, (1 Cr & Cb sample per 2x2 Y & A samples)
		NV12            = PIX_FMT_NV12,        ///< Planar YUV 4:2:0, 12bpp, 1 plane for Y and 1 for UV
		NV21            = PIX_FMT_NV21,        ///< as above, but U and V bytes are swapped
		YUV420P16LE     = PIX_FMT_YUV420P16LE, ///< Planar YUV 4:2:0, 24bpp, (1 Cr & Cb sample per 2x2 Y samples), little-endian
		YUV420P16BE     = PIX_FMT_YUV420P16BE, ///< Planar YUV 4:2:0, 24bpp, (1 Cr & Cb sample per 2x2 Y samples), big-endian
		YUV420P16       = PIX_FMT_YUV420P16,   ///< Planar YUV 4:2:0, 24bpp, (1 Cr & Cb sample per 2x2 Y samples), in cpu endianess

		YUV422P         = PIX_FMT_YUV422P,     ///< Planar YUV 4:2:2, 16bpp, (1 Cr & Cb sample per 2x1 Y samples)
		YUVJ422P        = PIX_FMT_YUVJ422P,    ///< Planar YUV 4:2:2, 16bpp, full scale (jpeg)
		YUYV422         = PIX_FMT_YUYV422,     ///< Packed YUV 4:2:2, 16bpp, Y0 Cb Y1 Cr
		UYVY422         = PIX_FMT_UYVY422,     ///< Packed YUV 4:2:2, 16bpp, Cb Y0 Cr Y1
		YUV422P16LE     = PIX_FMT_YUV422P16LE, ///< Planar YUV 4:2:2, 32bpp, (1 Cr & Cb sample per 2x1 Y samples), little-endian
		YUV422P16BE     = PIX_FMT_YUV422P16BE, ///< Planar YUV 4:2:2, 32bpp, (1 Cr & Cb sample per 2x1 Y samples), big-endian
		YUV422P16       = PIX_FMT_YUV422P16,   ///< Planar YUV 4:2:2, 32bpp, (1 Cr & Cb sample per 2x1 Y samples), in cpu endianess

		YUV440P         = PIX_FMT_YUV440P,     ///< Planar YUV 4:4:0 (1 Cr & Cb sample per 1x2 Y samples)
		YUVJ440P        = PIX_FMT_YUVJ440P,    ///< Planar YUV 4:4:0 full scale (jpeg)

		YUV444P         = PIX_FMT_YUV444P,     ///< Planar YUV 4:4:4, 24bpp, (1 Cr & Cb sample per 1x1 Y samples)
		YUVJ444P        = PIX_FMT_YUVJ444P,    ///< Planar YUV 4:4:4, 24bpp, full scale (jpeg)
		YUV444P16LE     = PIX_FMT_YUV444P16LE, ///< Planar YUV 4:4:4, 48bpp, (1 Cr & Cb sample per 1x1 Y samples), little-endian
		YUV444P16BE     = PIX_FMT_YUV444P16BE, ///< Planar YUV 4:4:4, 48bpp, (1 Cr & Cb sample per 1x1 Y samples), big-endian
		YUV444P16       = PIX_FMT_YUV444P16,   ///< Planar YUV 4:4:4, 48bpp, (1 Cr & Cb sample per 1x1 Y samples), in cpu endianess

		XvmcMpeg2Mc     = PIX_FMT_XVMC_MPEG2_MC,///< XVideo Motion Acceleration via common packet passing(xvmc_render.h)
		XvmcMpeg2Idct   = PIX_FMT_XVMC_MPEG2_IDCT,

		VDPAU_H264      = PIX_FMT_VDPAU_H264,  ///< H264 HW decoding with VDPAU, data[0] contains a vdpau_render_state struct which contains the bitstream of the slices as well as various fields extracted from headers
		VDPAU_MPEG1     = PIX_FMT_VDPAU_MPEG1, ///< MPEG-1 HW decoding with VDPAU, data[0] contains a vdpau_render_state struct which contains the bitstream of the slices as well as various fields extracted from headers
		VDPAU_MPEG2     = PIX_FMT_VDPAU_MPEG2, ///< MPEG-2 HW decoding with VDPAU, data[0] contains a vdpau_render_state struct which contains the bitstream of the slices as well as various fields extracted from headers
		VDPAU_WMV3      = PIX_FMT_VDPAU_WMV3,  ///< WMV3 HW decoding with VDPAU, data[0] contains a vdpau_render_state struct which contains the bitstream of the slices as well as various fields extracted from headers
		VDPAU_VC1       = PIX_FMT_VDPAU_VC1,   ///< VC-1 HW decoding with VDPAU, data[0] contains a vdpau_render_state struct which contains the bitstream of the slices as well as various fields extracted from headers
		VDPAU_MPEG4     = PIX_FMT_VDPAU_MPEG4, ///< MPEG4 HW decoding with VDPAU, data[0] contains a vdpau_render_state struct which contains the bitstream of the slices as well as various fields extracted from headers

		VAAPI_MOCO      = PIX_FMT_VAAPI_MOCO,  ///< HW acceleration through VA API at motion compensation entry-point, Picture.data[3] contains a vaapi_render_state struct which contains macroblocks as well as various fields extracted from headers
		VAAPI_IDCT      = PIX_FMT_VAAPI_IDCT,  ///< HW acceleration through VA API at IDCT entry-point, Picture.data[3] contains a vaapi_render_state struct which contains fields extracted from headers
		VAAPI_VLD       = PIX_FMT_VAAPI_VLD,   ///< HW decoding through VA API, Picture.data[3] contains a vaapi_render_state struct which contains the bitstream of the slices as well as various fields extracted from headers

		NB              = PIX_FMT_NB,          ///< number of pixel formats, DO NOT USE THIS if you want to link with shared libav* because the number of formats might differ between versions
	};

	///<summary>
	///Flags for use with sws_scale.
	///<remark>Copied verbatim from swscale.h v0.6.1.</remark>
	///</summary>
	[Flags] public enum class SwsFlags {
		FastBilinear    = SWS_FAST_BILINEAR,
		Bilinear        = SWS_BILINEAR,
		Bicubic         = SWS_BICUBIC,
		X               = SWS_X,
		Point           = SWS_POINT,
		Area            = SWS_AREA,
		Bibublin        = SWS_BICUBLIN,
		Gauss           = SWS_GAUSS,
		Sinc            = SWS_SINC,
		Lanczos         = SWS_LANCZOS,
		Spline          = SWS_SPLINE,

		SRC_V_CHR_DROP_MASK = SWS_SRC_V_CHR_DROP_MASK,

		PARAM_DEFAULT = SWS_PARAM_DEFAULT,

		PrintInfo = SWS_PRINT_INFO,

		//the following 3 flags are not completely implemented
		//internal chrominace subsampling info
		FULL_CHR_H_INT = SWS_FULL_CHR_H_INT,
		//input subsampling info
		FULL_CHR_H_INP = SWS_FULL_CHR_H_INP,
		DIRECT_BGR = SWS_DIRECT_BGR,
		ACCURATE_RND = SWS_ACCURATE_RND,
		BITEXACT = SWS_BITEXACT,

		CPU_CAPS_MMX = SWS_CPU_CAPS_MMX,
		CPU_CAPS_MMX2 = SWS_CPU_CAPS_MMX2,
		CPU_CAPS_3DNOW = SWS_CPU_CAPS_3DNOW,
		CPU_CAPS_ALTIVEC = SWS_CPU_CAPS_ALTIVEC,
		CPU_CAPS_BFIN = SWS_CPU_CAPS_BFIN,
	};

} // namespace LibAvWrap