// LibAvWrap -- .NET wrapper for the libav* libraries
// --------------------------------------------------
// By Christian Richardt <christian@richardt.name>
// (based on AvDn 0.7, www.informikon.com/tutorials/)
//
// $Id: FormatContext.h 436 2010-10-15 16:26:38Z cr333 $

#pragma once

using namespace System;
using namespace System::Text;
using namespace System::Runtime::InteropServices;

extern "C" {
#include "libavformat/avformat.h"
}

#include "Enums.h"
#include "Wrapper.h"

///<summary>Wrapper for some lavc and lavf structures.</summary>
namespace LibAvWrap
{
	// forward declarations
	ref class LibAvByteIOContext;
	ref class LibAvCodecContext;
	ref class LibAvOutputFormat;
	ref class LibAvPacket;
	ref class LibAvStream;

	///<summary>Wrapper for the AVFormatContext structure.</summary>
	public ref class LibAvFormatContext : Wrapper<AVFormatContext>
	{
		// flag to decide if we need to call av_free in the destructor
		bool need_to_free;

// constructors and destructor --------------------------------------------------------------------

	public :
		///<summary>Constructs an empty format context.</summary>
		LibAvFormatContext();

		///<summary>Constructs a format context from a given filename.</summary>
		LibAvFormatContext(String^ filename);

		///<summary>Constructs a format context from a given native pointer.</summary>
		LibAvFormatContext(IntPtr ptr);

		///<summary>Destructor.</summary>
		~LibAvFormatContext();
		
// member functions -------------------------------------------------------------------------------

		///<summary>Closes the input file stream.</summary>
		void CloseInputFile();

		///<summary>Creates a new stream.</summary>
		LibAvStream^ CreateNewStream(int nb);

		///<summary>Dump some information about a format.</summary>
		void DumpFormat(int index, String^ url, bool isOutput);

		///<summary>Returns the first video stream that is found.</summary>
		int FindFirstVideoStream();

		///<summary>Returns the stream info.</summary>
		int FindStreamInfo();

		///<summary>Gets the codec context for a stream.</summary>
		LibAvCodecContext^ GetCodecContextForStream(int i);

		///<summary>Returns the codec type for a stream.</summary>
		LibAvWrap::MediaType GetCodecTypeForStream(int i);

		//int OpenInputFile(String* filename ) { 
		//   return av_open_input_file(&pFormatCtx, filename, NULL, 0, NULL);
		//}

		///<summary>Reads a frame from a packet.</summary>
		int ReadFrame(LibAvPacket^ p);

		///<summary>Set some parameters (use with null only so far).</summary>
		int SetParameters(IntPtr ptr);

		///<summary>Returns a pointer to "the bytes" as a ByteIOContext.</summary>
		LibAvByteIOContext^ GetPb();

		///<summary>Writes a frame from a packet.</summary>
		int WriteFrame(LibAvPacket^ p);

		///<summary>Write the header for a stream given the current AVFormatContext.</summary>
		void WriteHeader();

		///<summary>Write the trailer for a stream given the current AVFormatContext.</summary>
		void WriteTrailer();

// properties -------------------------------------------------------------------------------------

		///<summary>Input or output filename.</summary>
		property String^ Filename { String^ get(); void set(String^ filename); }

		///<summary>Number of streams field of the AVFormatContext structure.</summary>
		property int NbStreams { int get(); }

		///<summary>Output format field of the AVFormatContext structure.</summary>
		property LibAvOutputFormat^ OutputFormat { LibAvOutputFormat^ get(); void set(LibAvOutputFormat^ format); }
	};

} // namespace LibAvWrap