/*
This source code is part of the Floating Textures test program, which
is a derivative of the complete Floating Textures program containing
the offline version with prereconstructed geometry and prerecorded
video. 
For more information on Floating Textures please have a look at the article

"Floating Textures" 
by M. Eisemann, B. De Decker, M. Magnor, 
P. Bekaert, E. de Aguiar, N. Ahmed, C. Theobalt, A. Sellent
Computer Graphics Forum (Proc. Eurographics EG'08), vol. 27, no. 2, April 2008 


Copyright (C) 2008 Martin Eisemann

This program is free software; you can redistribute it and/or modify it under 
the terms of the GNU General Public License as published by the Free Software 
Foundation; either version 3 of the License, or (at your option) any later 
version.

This program is distributed in the hope that it will be useful, but WITHOUT 
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS 
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with 
this program; if not, see <http://www.gnu.org/licenses/>.
*/

uniform sampler2D source0;
uniform sampler2D source1;
uniform sampler2D source2;

uniform sampler2D opticalflow0; // I0 <-> I1
uniform sampler2D opticalflow1; // I0 <-> I2
uniform sampler2D opticalflow2; // I1 <-> I2

uniform sampler2D weightMap;

uniform vec2 dxy; // step size, depending on texture resolution NxM -> (1/N,1/M)
                  // dxy.x -> x-direction
                  // dxy.y -> y-direction

const vec4 ZERO4 = vec4(0.0, 0.0, 0.0, 0.0);
const vec3 ZERO3 = vec3(0.0, 0.0, 0.0);

void main() {  
  
  vec2 tC = gl_TexCoord[0].xy;
  
  // get the weights //
  vec3 weights = texture2D(weightMap, tC).xyz;

  // calculate the warped coordinates, depending on the flow fields
  vec4 flow0_1 = texture2D(opticalflow0, tC);
  vec4 flow0_2 = texture2D(opticalflow1, tC);
  vec4 flow1_2 = texture2D(opticalflow2, tC);

  vec2 warpTC[3];
  // warp for image 0 //
  warpTC[0]  = tC - vec2(weights.y, weights.y) * dxy * flow0_1.xy;
  warpTC[0] -= vec2(weights.z, weights.z) * dxy * flow0_2.xy;
  // warp for image 1 //
  warpTC[1]  = tC - vec2(weights.x, weights.x) * dxy * flow0_1.zw;
  warpTC[1] -= vec2(weights.z, weights.z) * dxy * flow1_2.xy;
  // warp for image 2 //
  warpTC[2]  = tC - vec2(weights.x, weights.x) * dxy * flow0_2.zw;
  warpTC[2] -= vec2(weights.y, weights.y) * dxy * flow1_2.zw;
 

  vec4 color[3];
  color[0] = texture2D(source0, warpTC[0]);
  color[1] = texture2D(source1, warpTC[1]);
  color[2] = texture2D(source2, warpTC[2]);  


  // prevent the foreground to be warped into the background //
  float sum = weights.x + weights.y + weights.z;
  if(sum < 0.0001)
    gl_FragColor = vec4(ZERO3, 1.0);

  // prevent the background to be warped into the object //
  else{
    if(color[0].a == 0.0)
      weights.x = 0.0;
    if(color[1].a == 0.0)
      weights.y = 0.0;
    if(color[2].a == 0.0)
      weights.z = 0.0;
    
    
    weights = weights / (sum + 0.0001);
    
    
    gl_FragColor = ((weights.x * color[0]) +
		    (weights.y * color[1]) +
		    (weights.z * color[2]));
  }
}

