﻿// $Id: JointBilateralFilter.frag 499 2010-11-17 16:29:28Z cr333 $
#extension GL_EXT_gpu_shader4 : enable

// input texture
uniform sampler2D edges;
uniform sampler2D values;

// parameters
uniform float spatialSigma;
uniform float rangeSigma;

void main()
{
	// look up texture size once and reuse it later,
	// assuming every texture has the same size
	vec2 size = vec2(textureSize2D(edges, 0));

	// precompute 1 / sigma^2
	float iSigmaS = 1.0 / (2.0 * spatialSigma * spatialSigma);
	float iSigmaR = 1.0 / (2.0 * rangeSigma   * rangeSigma);

	// truncate Gaussian after 2 sigma
	int radius = int(ceil(2.0 * spatialSigma));

	// colour value of edge image at the central pixel
	vec3 edgeCentre = texture2D(edges, (gl_FragCoord.xy + 0.5) / size).rgb;
	
	// homogeneous accumulator of values, i.e. (wr, wg, wb, w)
	vec4 acc = vec4(0);

	for(int dy = -radius; dy <= radius; dy++)
	{
		for(int dx = -radius; dx <= radius; dx++)
		{
			vec2 coords = gl_FragCoord.xy + vec2(dx, dy);

			// colour values at the current pixel
			vec3 currentEdge  = texture2D(edges,  (coords + 0.5) / size).rgb;
			vec3 currentValue = texture2D(values, (coords + 0.5) / size).rgb;

			// compute pixel weight, using exp(a) * exp(b) = exp(a + b);
			float w = 0.0;

			// spatial weight
			vec2 deltaS = vec2(dx, dy);
			w += iSigmaS * dot(deltaS, deltaS);
			
			// range weight
			vec3 deltaR = currentEdge - edgeCentre;
			w += iSigmaR * dot(deltaR, deltaR);

			acc += exp(-w) * vec4(currentValue, 1.0);
		}
	}

	// filtered colour (black if there were no valid data samples in the kernel)
	if(acc.w <= 0.0) gl_FragColor = vec4(0.0, 0.0, 0.0, 1.0);
	else             gl_FragColor = vec4(acc.xyz / acc.w, 1.0);
}