﻿// $Id: Exceptions.cs 436 2010-10-15 16:26:38Z cr333 $
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using VideoLib.Frames;

namespace VideoLib
{
	/// <summary>Represents an exception due to missing pins.</summary>
	[global::System.Serializable]
	public class PinNotFoundException : Exception
	{
		/// <summary>Initializes a new instance of the <see cref="PinNotFoundException"/> class.</summary>
		public PinNotFoundException() { }

		/// <summary>Initializes a new instance of the <see cref="PinNotFoundException"/> class.</summary>
		/// <param name="message">The message.</param>
		public PinNotFoundException(string message) : base(message) { }

		/// <summary>Initializes a new instance of the <see cref="PinNotFoundException"/> class.</summary>
		/// <param name="message">The message.</param>
		/// <param name="inner">The inner exception.</param>
		public PinNotFoundException(string message, Exception inner) : base(message, inner) { }

		/// <summary>Initializes a new instance of the <see cref="PinNotFoundException"/> class.</summary>
		/// <param name="info">The <see cref="T:System.Runtime.Serialization.SerializationInfo"/> that holds the serialized object data about the exception being thrown.</param>
		/// <param name="context">The <see cref="T:System.Runtime.Serialization.StreamingContext"/> that contains contextual information about the source or destination.</param>
		/// <exception cref="T:System.ArgumentNullException">The <paramref name="info"/> parameter is null. </exception>
		/// <exception cref="T:System.Runtime.Serialization.SerializationException">The class name is null or <see cref="P:System.Exception.HResult"/> is zero (0). </exception>
		protected PinNotFoundException(
		  System.Runtime.Serialization.SerializationInfo info,
		  System.Runtime.Serialization.StreamingContext context)
			: base(info, context) { }
	}

	/// <summary>Represents an exception due to unsupported frame types.</summary>
	[global::System.Serializable]
	public class FrameNotSupportedException : Exception
	{
		/// <summary>Initializes a new instance of the <see cref="FrameNotSupportedException"/> class.</summary>
		public FrameNotSupportedException() { }

		/// <summary>Initializes a new instance of the <see cref="FrameNotSupportedException"/> class.</summary>
		/// <param name="message">The message.</param>
		public FrameNotSupportedException(string message) : base(message) { }

		/// <summary>Initializes a new instance of the <see cref="FrameNotSupportedException"/> class.</summary>
		/// <param name="frame">The offending frame.</param>
		public FrameNotSupportedException(Frame frame) : base(String.Format("Frame of type \"{0}\" is not supported here.", frame.ToString())) { }

		/// <summary>Initializes a new instance of the <see cref="FrameNotSupportedException"/> class.</summary>
		/// <param name="message">The message.</param>
		/// <param name="inner">The inner exception.</param>
		public FrameNotSupportedException(string message, Exception inner) : base(message, inner) { }

		/// <summary>Initializes a new instance of the <see cref="FrameNotSupportedException"/> class.</summary>
		/// <param name="info">The <see cref="T:System.Runtime.Serialization.SerializationInfo"/> that holds the serialized object data about the exception being thrown.</param>
		/// <param name="context">The <see cref="T:System.Runtime.Serialization.StreamingContext"/> that contains contextual information about the source or destination.</param>
		/// <exception cref="T:System.ArgumentNullException">The <paramref name="info"/> parameter is null. </exception>
		/// <exception cref="T:System.Runtime.Serialization.SerializationException">The class name is null or <see cref="P:System.Exception.HResult"/> is zero (0). </exception>
		protected FrameNotSupportedException(
		  System.Runtime.Serialization.SerializationInfo info,
		  System.Runtime.Serialization.StreamingContext context)
			: base(info, context) { }
	}
}
