﻿// $Id: CudaWorkQueue.cs 440 2010-10-18 11:49:09Z cr333 $
// Partially based on a Part II project by Rubin Xu (2010).
using System.Collections.Generic;
using System.Threading;
using VideoLib.Frames;

namespace VideoLib.Scheduling
{
	/// <summary>Implements a work queue just for CUDA-based nodes, which are marked using the ICudaNode interface.</summary>
	/// <remarks>Should be applied before any CPU-based and frame transport work queues.</remarks>
	public class CudaWorkQueue : WorkQueue
	{
		/// <summary>Starts all worker threads associated with this work queue.</summary>
		/// <returns>Collection of all started worker threads.</returns>
		public override IEnumerable<Thread> StartWorkers()
		{
			// start just a single thread, as the CUDA context is thread-specific
			StartWorker(0, "CudaWorker");
			return workers;
		}

		/// <summary>Handles initialisation of nodes.</summary>
		/// <param name="node">The node to be initialised.</param>
		/// <returns>True, if and only if the node has been initialised.</returns>
		public override bool HandleNodeInitEvent(INode node)
		{
			if (node is ICudaNode) // only initialise CUDA nodes
				return HandleNodeInitEvent(node, Ready);

			return false; // event not handled
		}

		/// <summary>Handles nodes that become ready to run.</summary>
		/// <param name="node">The ready node.</param>
		/// <returns>True, if and only if the event was handled.</returns>
		public override bool HandleNodeReadyEvent(INode node)
		{
			if (node is ICudaNode) // add CUDA nodes to CUDA-only queue
				return HandleNodeReadyEvent(node, Scheduler.Profiler, Ready);

			return false; // event not handled
		}
	}
}