﻿// $Id: IntrinsicParameters.cs 411 2010-10-04 19:39:54Z cr333 $

using System;
namespace VideoLib.Vision
{
	/// <summary>Collection of intrinsic camera parameters.</summary>
	public class IntrinsicParameters
	{
		/// <summary>Gets or sets the focal length along x.</summary>
		public float Fx { get; set; }

		/// <summary>Gets or sets the focal length along y.</summary>
		public float Fy { get; set; }

		/// <summary>Gets or sets the pricipal point's x coordinate.</summary>
		public float Cx { get; set; }

		/// <summary>Gets or sets the pricipal point's y coordinate.</summary>
		public float Cy { get; set; }

		/// <summary>Gets or sets the skew coefficient.</summary>
		public float Alpha { get; set; }

		/// <summary>Gets or sets the first radial distortion coefficient.</summary>
		public float K1 { get; set; }

		/// <summary>Gets or sets the second radial distortion coefficient.</summary>
		public float K2 { get; set; }

		/// <summary>Gets or sets the first tangential distortion coefficient.</summary>
		public float K3 { get; set; }

		/// <summary>Gets or sets the second tangential distortion coefficient.</summary>
		public float K4 { get; set; }

		/// <summary>Gets or sets the third radial distortion coefficient.</summary>
		public float K5 { get; set; }

		// convenience properties -----------------------------------------------------------------------

		/// <summary>Gets or sets the principal point as a float[2].</summary>
		public float[] C {
			get { return new float[] { Cx, Cy }; }
			set
			{
				if (value.Length != 2) throw new ArgumentException("Expecting float[2].");
				Cx = value[0]; Cy = value[1];
			}
		}

		/// <summary>Sets both focal lengths to the same value.</summary>
		public float F { set { Fx = value; Fy = value; } }

		/// <summary>Gets or sets all lens distortion parameters (as per Bouguet).</summary>
		public float[] K
		{
			get { return new float[] { K1, K2, K3, K4, K5 }; }
			set
			{
				if (value.Length >= 1) K1 = value[0];
				if (value.Length >= 2) K2 = value[1];
				if (value.Length >= 3) K3 = value[2];
				if (value.Length >= 4) K4 = value[3];
				if (value.Length >= 5) K5 = value[4];
			}
		}
	}
}